package com.shixing.edit.widget;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.PointF;
import android.util.AttributeSet;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.ScaleGestureDetector;
import android.view.View;

import com.shixing.edit.data.EditData;
import com.shixing.edit.data.TrackSourceObject;
import com.shixing.edit.manager.ActionManager;
import com.shixing.edit.multitrack.TrackActionListener;
import com.shixing.edit.multitrack.TrackActionManager;
import com.shixing.edit.multitrack.TrackViewListener;
import com.shixing.edit.utils.RotationGestureDetector;
import com.shixing.sxedit.internal.EditManager;
import com.shixing.sxedit.internal.Effect;
import com.shixing.sxedit.internal.MediaTrack;
import com.shixing.sxedit.internal.TextTrack;
import com.shixing.sxedit.internal.Track;
import com.shixing.sxedit.internal.TrackGroup;

import java.lang.reflect.Field;
import java.util.List;

import static com.shixing.sxedit.track.SXTrack.SX_TRACK_TYPE_MEDIA;
import static com.shixing.sxedit.track.SXTrack.SX_TRACK_TYPE_STICKER;
import static com.shixing.sxedit.track.SXTrack.SX_TRACK_TYPE_TEXT;

public class SurfaceOverlay extends View implements TrackViewListener, TrackActionListener {
    private static final String TAG = "SurfaceOverlay";

    private GestureDetector mGestureDetector;
    private ScaleGestureDetector mScaleDetector;
    private RotationGestureDetector mRotationDetector;

    private AspectRatioFrameLayout mSurfaceContainer;
    private EditData mEditData;
    private Paint mPaint;

    private TextBoxView textBoxView = new TextBoxView();
    private TrackFrame trackFrame = new TrackFrame();
    private MaskView maskView = new MaskView();

    public static final int MOD_NONE = 0;
    public static final int MOD_CLIP = 1;
    public static final int MOD_TEXT = 2;
    public static final int MOD_PIP = 3;
    public static final int MOD_MASK = 4;

    private int currentMod = MOD_NONE;
    private boolean editing;

    public SurfaceOverlay(Context context) {
        this(context, null);
    }

    public SurfaceOverlay(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public SurfaceOverlay(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);

        mGestureDetector = new GestureDetector(getContext(), new GestureListener());
        mScaleDetector = new ScaleGestureDetector(getContext(), new ScaleListener());
        mRotationDetector = new RotationGestureDetector(new RotationListener());

        try {
            Field minSpan = mScaleDetector.getClass().getDeclaredField("mMinSpan");
            minSpan.setAccessible(true);
            minSpan.set(mScaleDetector, 20);
        } catch (NoSuchFieldException | IllegalAccessException e) {
            e.printStackTrace();
        }

        mEditData = EditData.getInstance();
        mPaint = new Paint();
        mPaint.setColor(Color.RED);
        mPaint.setStyle(Paint.Style.STROKE);
        mPaint.setStrokeWidth(4);

        textBoxView.initIcon(getResources());
        textBoxView.setActionListener(new TextBoxView.ActionListener() {
            @Override
            public void onClickEdit(Track track) {
                if (listener != null) {
                    listener.onEditTrack(track);
                }
            }

            @Override
            public void onClickDelete(Track track) {
                ActionManager.getInstance().getListener().removeTrack(track);
                TrackActionManager.getInstance().cancelSelectTrackByClick();
                TrackActionManager.getInstance().removeTrack(track);
                ActionManager.getInstance().getListener().updateCurrentFrame();
            }

            @Override
            public void onClickCopy(Track track) {
                Track copyTrack = ActionManager.getInstance().getListener().copyTrack(track);
                if (copyTrack != null) {
                    TrackActionManager.getInstance().addTrackGroup(ActionManager.getInstance().getListener().getTextStickerGroups(), SX_TRACK_TYPE_TEXT);
                    ActionManager.getInstance().getListener().updateCurrentFrame(new EditManager.UpdateListener() {
                        @Override
                        public void onUpdateFinish() {
                            TrackActionManager.getInstance().onTrackSelect(copyTrack, true);
                        }
                    });
                }
            }
        });
    }

    public void setSurfaceContainer(AspectRatioFrameLayout surfaceContainer) {
        mSurfaceContainer = surfaceContainer;
    }

    private OverlayListener listener;

    public void onFrameChanged() {
        textBoxView.updateTrackSize();
    }

    public interface OverlayListener {
        void onEditTrack(Track track);
    }

    public void setListener(OverlayListener listener) {
        this.listener = listener;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        canvas.save();
        canvas.concat(mSurfaceContainer.mMatrix);

        switch (currentMod) {
            case MOD_NONE:
                break;
            case MOD_CLIP:
            case MOD_PIP:
                trackFrame.drawFrame(canvas, mPaint);
                break;
            case MOD_TEXT:
                textBoxView.draw(canvas, mPaint);
            case MOD_MASK:
                maskView.draw(canvas);
        }
        canvas.restore();
    }

    private Matrix invertMatrix = new Matrix();
    private boolean handleText = false;

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (currentMod == MOD_NONE) {
            return false;
        }

        mSurfaceContainer.mMatrix.invert(invertMatrix);
        event.transform(invertMatrix);

        switch (event.getActionMasked()) {
            case MotionEvent.ACTION_DOWN:
                if (textBoxView.isHandle(event.getX(), event.getY())) {
                    handleText = true;
                    setCurrentMod(MOD_TEXT);
                    return true;
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (handleText) {
                    textBoxView.handleMove(event.getX(), event.getY());
                    mEditData.editManager.updateCurrentFrame();
                    invalidate();
                    return true;
                }
                break;
            case MotionEvent.ACTION_UP:
                handleText = false;
        }

        mGestureDetector.onTouchEvent(event);
        mScaleDetector.onTouchEvent(event);
        mRotationDetector.onTouchEvent(event);

        return true;
    }

    public void setCurrentMod(int currentMod) {
        this.currentMod = currentMod;
        invalidate();
    }

    public void showTrackFrame(Track track) {
        trackFrame.setTrack(track);
        invalidate();
    }

    public void showTextFrame(Track track) {
        textBoxView.setTrack(track);
        invalidate();
    }

    public void showMask(Track track) {
        maskView.setTrack(track);
        invalidate();
    }

    public void setEditing(boolean editing) {
        this.editing = editing;
        textBoxView.setEditing(editing);
        invalidate();
    }

    @Override
    public void updateMaxDuration(double duration, boolean isActionUp) {

    }

    @Override
    public void updateScaleSize(float scale) {

    }

    @Override
    public boolean isClicked() {

        return false;
    }

    @Override
    public void cancelSelectTrackByClick() {
    }

    @Override
    public void cancelSelectEffectByClick() {

    }


    @Override
    public void updateTrack(Track track) {

    }

    @Override
    public void updateMainTrack(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffect(Effect effect) {

    }

    @Override
    public void updateTrackDuration(Track track) {

    }

    @Override
    public void updateMainTrackDuration(MediaTrack mainTrack) {

    }


    @Override
    public void updateEffectDuration(Effect effect) {

    }

    @Override
    public void removeTrack(Track track) {

    }

    @Override
    public void removeMainTrack(MediaTrack mediaTrack) {

    }


    @Override
    public void removeEffect(Effect effect) {

    }

    @Override
    public void addTrack(Track track) {

    }

    @Override
    public void addMainTrack(List<MediaTrack> mainTracks) {

    }

    @Override
    public void addEffect(Effect effect) {

    }

    @Override
    public void addTrackGroup(List<TrackGroup> trackGroups, int trackType) {

    }

    @Override
    public void addEffectList(List<Effect> effectList, int trackType) {

    }

    @Override
    public void addTrackBeanSource(TrackSourceObject sourceObject) {

    }

    @Override
    public void onProgressChanged(double time) {
        trackFrame.setCurrentTime(time);
        textBoxView.setCurrentTime(time);
        invalidate();
    }

    @Override
    public boolean canBeAddEffect(Effect effect) {
        return false;
    }

    @Override
    public void onMovingUp() {

    }

    @Override
    public void onMovingDown() {

    }

    @Override
    public void onTouchScroll() {

    }

    @Override
    public void onTrackSelect(Track track,boolean isSwitch) {
        switch (track.type()) {
            case SX_TRACK_TYPE_STICKER:
            case SX_TRACK_TYPE_TEXT:
                setCurrentMod(MOD_TEXT);
                showTextFrame(track);
                break;
            case SX_TRACK_TYPE_MEDIA:
//                MediaTrack mediaTrack = (MediaTrack) track;
//                if (mediaTrack.inMinTrackGroup()) {
//                    setCurrentMod(MOD_CLIP);
//                    showTrackFrame(mediaTrack);
//                } else {
//                    setCurrentMod(MOD_PIP);
//                }
                showTrackFrame(track);
                break;
        }
    }

    @Override
    public void onEffectSelect(Effect effect,boolean isSwitch) {

    }

    @Override
    public void onTrackUnSelect(Track track) {
        trackFrame.setTrack(null);
        textBoxView.setTrack(null);
        maskView.setTrack(null);
        invalidate();
    }

    @Override
    public void onEffectUnSelect(Effect effect) {

    }

    @Override
    public void onClickTransition(Track track) {

    }

    @Override
    public void hideDetailFragment() {

    }

    private class GestureListener extends GestureDetector.SimpleOnGestureListener {
        @Override
        public boolean onSingleTapConfirmed(MotionEvent e) {
            if (currentMod == MOD_TEXT) {
                if (textBoxView.onSingleTap(e.getX(), e.getY())) {
                    return true;
                }
                float x = e.getX();
                float y = e.getY();
                Track track = mEditData.editManager.hitPoint(new PointF(x, y), SX_TRACK_TYPE_STICKER | SX_TRACK_TYPE_TEXT);
                if (track == null && textBoxView.getTrack() != null && !editing) {
                    TrackActionManager.getInstance().onTrackUnSelect(textBoxView.getTrack());
                }
            }
            return super.onSingleTapConfirmed(e);
        }

        @Override
        public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) {
            switch (currentMod) {
                case MOD_CLIP:
                case MOD_PIP:
                    trackFrame.translate(-distanceX, -distanceY);
                    break;
                case MOD_TEXT:
                    textBoxView.translate(-distanceX, -distanceY);
                    break;
                case MOD_MASK:
                    maskView.translate(-distanceX, -distanceY);
                    break;
            }

            mEditData.editManager.updateCurrentFrame();
            invalidate();
            return true;
        }

        @Override
        public boolean onDown(MotionEvent e) {
            if (currentMod == MOD_TEXT) {
                float x = e.getX();
                float y = e.getY();
                Track track = mEditData.editManager.hitPoint(new PointF(x, y), SX_TRACK_TYPE_STICKER | SX_TRACK_TYPE_TEXT);

                Track old = textBoxView.getTrack();
                if (track != null && !track.equals(old)) {
                    boolean isSwitch = false;
                    if (old != null) {
                        if (old.type() == track.type()) {
                            isSwitch = true;
                        }
//
                    }
                    if (!isSwitch)
                    TrackActionManager.getInstance().cancelSelectTrackByClick();
                    TrackActionManager.getInstance().onTrackSelect(track, isSwitch);
                }
            } else if (currentMod == MOD_PIP) {
//                float x = e.getX();
//                float y = e.getY();
//                Track track = mEditData.editManager.hitPoint(new PointF(x, y), SX_TRACK_TYPE_MEDIA);
//
//                if (track != null && !track.equals(trackFrame.getTrack())) {
//                    TrackActionManager.getInstance().onTrackSelect(track, true);
//                }
            }
            return super.onDown(e);
        }
    }

    private class ScaleListener extends ScaleGestureDetector.SimpleOnScaleGestureListener {
        @Override
        public boolean onScale(ScaleGestureDetector detector) {
            float scaleFactor = detector.getScaleFactor();
            switch (currentMod) {
                case MOD_CLIP:
                case MOD_PIP:
                    trackFrame.scale(scaleFactor, scaleFactor);
                    break;
                case MOD_TEXT:
                    textBoxView.scale(scaleFactor, scaleFactor);
                    break;
                case MOD_MASK:
                    maskView.scale(scaleFactor, scaleFactor);
                    break;
            }

            mEditData.editManager.updateCurrentFrame();
            invalidate();
            return true;
        }
    }

    private class RotationListener extends RotationGestureDetector.SimpleOnRotationGestureListener {
        @Override
        public boolean onRotation(RotationGestureDetector rotationDetector) {
            switch (currentMod) {
                case MOD_CLIP:
                case MOD_PIP:
                    trackFrame.rotate(rotationDetector.getAngle());
                    break;
                case MOD_TEXT:
                    textBoxView.rotate(rotationDetector.getAngle());
                    break;
                case MOD_MASK:
                    maskView.rotate(rotationDetector.getAngle());
                    break;
            }

            mEditData.editManager.updateCurrentFrame();
            invalidate();
            return true;
        }
    }
}

